--merge additional tables into table arg1. Earlier tables take precedence when conflicts arise.
local function tmerge(t, ...)
	for num=1, select('#',...) do
		local mt = select(num, ...)
		for key, value in pairs(mt) do
			t[key] = t[key] or value
		end
	end
	return t
end
--round a number
local function round(num, places)
	return tonumber(string.format("%."..(places or 0).."f",num))
end
local debug = false
--[===[@debug@
debug=true
--@end-debug@]===]
local function dprint(...)
	if not debug then return end
	print("===SS DEBUG===")
	for i=1, select('#',...) do
		print((string.gsub((select(i, ...)),"|","||")))
	end
end
StatusSubscribe = CreateFrame("Frame", "StatusSubscribe")
local StatusSubscribe = StatusSubscribe
StatusSubscribe = tmerge(StatusSubscribe,{encounters={},uencounters={},triggers={},EncounterStatus={}})
--[[
	:RegisterEncounter(unique_id, lname, options, triggerdata, encountertriggers) allows a plugin to register itself with StatusSubscribe.
	A first return value of true means that the encounter was registered successfully. If the first return value is false, the second return value holds an error code and the third value any additional information. The error code is one of the following:
	((
		"ERR_DUPLICATE" - The unique id already exists.
		"ERR_NO_ID" - No unique ID was provided.
		"ERR_NO_LNAME" - No localized name was provided.
		"ERR_NO_OPTION" - No options table was provided.
		"ERR_NO_TRIGGER" - No enable trigger table was provided.
		"ERR_NO_ETRIGGER" - No in-encounter trigger table was provided.
		"ERR_INVALID_OPT_TYPE" - One of the option table's values was of invalid type. arg3 holds the key.
		"ERR_INVALID_OPT_KEY" - One of the option table's keys was not recognized. arg3 holds the key.
		"ERR_NO_TRIGGER_EVENTS" - There are no enable trigger events available.
		"ERR_NO_TRIGGER_FILTER" - There is no enable trigger filter available.
		"ERR_NO_PHASES" - The encounter has no phases.
	))
	The StatusSubscribe.locale table holds localized error strings.
	unique_id is an unique string that identifies each encounter. this is used for addon communication if the target has the encounter available in their database.
	lname is the localized name of the encounter. this is sent to subscribers if their addon version does not have any encounter with matching unique_id
	options is a table containing the encounter's settings. Valid keys:
	options = {
		healthtrack = {[BossID]=true, [AddID]=false} <-- Format of this table is [npcID] = sticky. If a unit is sticky, it keeps displaying even after it dies.
		manatrack = {[BossID]=true, [AddID]=false} <-- Whether to track mana for the units specified in healthtrack. Defaults to false.
		phases = {[1]="Engaged", [2]="Enrage"} <-- Phases this encounter has. Format is [id] = localized_name. The localized name is only provided if
	}
	triggerdata is a table containing information about always-registered events. it contains two elements:
	triggerdata = {
		events = {"CHAT_MSG_MONSTER_YELL", "COMBAT_LOG_EVENT_UNFILTERED"} <-- this table contains all the events this encounter listens to
		filter = function(event, msg) if event == "CHAT_MSG_YELL" and msg == "I have been engaged! Minions, come to my aid!" then return true end end <-- this function is called when any of the above events fire while no encounter is engaged. If it returns true, the encounter becomes engaged
	}
	encountertriggers is a table containing all events and respective functions that should be called while the encounter is in progress. self and event are ommitted, arg1 is the first event arg. these functions can call special semi-global functions to affect the course of the encounter:
	((
		DisableEncounter() - Disables the encounter. Takes no arguments.
		EnterPhase(id) - Takes a phase ID as argument. Notifies subscribers that the encounter has entered this phase.
		IsInPhase(id) - Returns a bool indicating whether the encounter is in the specified phase
		TimerCreate(name, type, value) - Creates a timer/count. Valid types:
		((
			"ucount" - Creates a counter that starts at 0 and is displayed as "curvalue/value" <-- value is the third argument to TimerCreate
			"dcount" - Creates a counter that starts at value and is displayed as "value/value" <-- value is the third argument to TimerCreate
			"utime" - Creates a timer that starts at 0 and is displayed as "mm:ss/mm:ss". The right value is the third argument to TimerCreate (seconds)
			"dtime" - Creates a timer that starts at value and is displayed as "mm:ss/mm:ss". The right value is the third argument to TimerCreate (seconds)
		))
		TimerSet(name, value) - Sets the timer with the name name to the value value.
		TimerKill(name) - Removes a timer. name is the name used to create the timer earlier.
		TimerValue(name) - Returns the value of the timer
	))
	encountertriggers = {
		["CHAT_MSG_YELL"] = function(msg) if event == "CHAT_MSG_YELL" then if msg == "No! I have been defeated! Impossible!" then DisableEncounter() elseif msg == "No! Fear my wrath!" then EnterPhase(2, "Enrage") end end end
		["ENCOUNTER_ENGAGE"] = function() EnterPhase(1) end
		["ALL_OUT_OF_COMBAT"] = function() DisableEncounter() end
	}
	A special member function of encountertriggers, "ENCOUNTER_ENGAGE", is called when the encounter engages. Use this to set the first phase the boss enters.
	Another special function, "ALL_OUT_OF_COMBAT", is called when all raid members are either dead or out of combat.
--]]
function StatusSubscribe:RegisterEncounter(unique_id, lname, options, triggerdata, encountertriggers)
	if not unique_id then
		return false, "ERR_NO_ID"
	end
	if not lname then
		return false, "ERR_NO_LNAME"
	end
	if not options then
		return false, "ERR_NO_OPTION"
	end
	if not triggerdata then
		return false, "ERR_NO_TRIGGER"
	end
	if not encountertriggers then
		return false, "ERR_NO_ETRIGGER"
	end
	if self.encounters[unique_id] then
		return false, "ERR_DUPLICATE"
	end
	if (not triggerdata.events) or (type(triggerdata.events) ~= "table") then
		return false, "ERR_NO_TRIGGER_EVENTS"
	end
	if (not options.phases) then
		return false, "ERR_NO_PHASES"
	end
	local hasPhase
	for i in pairs(options.phases) do
		hasPhase = true
		break
	end
	if not hasPhase then
		return false, "ERR_NO_PHASES"
	end
	self.encounters[unique_id] = {lname = lname,hooks={}}
	for optkey, value in pairs(options) do
		if optkey == "healthtrack" then
			if type(value) ~= "table" then
				return false, "ERR_INVALID_OPT_TYPE", optkey
			end
			self.encounters[unique_id].health = value
		elseif optkey == "manatrack" then
			if type(value) ~= "table" then
				return false, "ERR_INVALID_OPT_TYPE", optkey
			end
			self.encounters[unique_id].mana = value
		elseif optkey == "phases" then
			if type(value) ~= "table" then
				return false, "ERR_INVALID_OPT_TYPE", optkey
			else
				self.encounters[unique_id].phases = value
			end
		else
			return false, "ERR_INVALID_OPT_KEY", optkey
		end
	end
	for i, event in ipairs(triggerdata.events) do
		if not self.triggers[event] then
			self.triggers[event] = {}
			self:RegisterEvent(event)
		end
		tinsert(self.triggers[event], unique_id)
	end
	self.encounters[unique_id].engage = triggerdata.filter
	local funcenv = setmetatable({DisableEncounter=function() StatusSubscribe:DisengageEncounter(unique_id) end, EnterPhase=function(id) StatusSubscribe:EnterPhase(unique_id, id) end,IsInPhase=function(id) return StatusSubscribe:IsInPhase(unique_id,id) end, TimerCreate=function(name,type,value) StatusSubscribe:CreateTimer(unique_id,name,type,value) end,TimerSet=function(name,value) StatusSubscribe:TimerSet(unique_id,name,value) end,TimerKill=function(name) StatusSubscribe:KillTimer(unique_id,name) end,TimerValue=function(name) return StatusSubscribe:GetTimerValue(unique_id,name) end}, {__index=function(t,v) return _G[v] end})
	for event, func in pairs(encountertriggers) do
		setfenv(func, funcenv)
		self.encounters[unique_id].hooks[event] = func
	end
	dprint("Registered encounter "..lname.."("..unique_id..") successfully.", "Encounter Summary:", "Encounter has "..#self.encounters[unique_id].phases.." phases.")
	return true
end

StatusSubscribe.wipecheck = CreateFrame("Frame")
StatusSubscribe.wipecheck:Hide()
StatusSubscribe.wipecheck:SetScript("OnUpdate", function(self,elapsed)
	self.elapsed = (self.elapsed or 0)+elapsed
	if self.elapsed >= 3 then
		self.elapsed = 0
		local prefix = ((GetNumRaidMembers() == 0) and "party") or "raid"
		local isNoCombat = true
		for c=0, math.max(GetNumRaidMembers(), GetNumPartyMembers()) do
			local uid= (c==0 and "player" or prefix..c)
			if UnitAffectingCombat(uid) and not UnitIsDeadOrGhost(uid) then
				isNoCombat = false
				break
			end
		end
		if isNoCombat and StatusSubscribe.OnAllOutOfCombat then
			StatusSubscribe.OnAllOutOfCombat()
		end
	end
end)

StaticPopupDialogs["STATUSSUBSCRIBE_INC"] = {
	button1=ACCEPT,
	button2=DECLINE,
	text="|cffdf7800%s|r wants to recieve your status updates.",
	sound="levelup2",
	timeout=30,
	OnAccept = function(self,data,data2)
		SendAddonMessage("StatusSubscribe", "subconfirm", "WHISPER", data)
		StatusSubscribe_subscribers = StatusSubscribe_subscribers or {}
		tinsert(StatusSubscribe_subscribers, data)
	end,
	whileDead = true,
	hideOnEscape = true,
}

StaticPopupDialogs["STATUSSUBSCRIBE_DELETE"]={
	button1=YES,
	button2=NO,
	text="Really delete subscription to |cffdf7800%s|r?",
	showAlert = true,
	OnAccept=function(self,data)
		SendAddonMessage("StatusSubscribe", "subdelete", "WHISPER", StatusSubscribe_subscriptions[data])
		if StatusSubscribe.gui.curview == data then
			StatusSubscribe:SetView(0)
		end
		tremove(StatusSubscribe_subscriptions, data)
		StatusSubscribe:UpdateSubscriptionTabs()
	end,
	whileDead=true,
	hideOnEscape=true,
	timeout=0,
}

StaticPopupDialogs["STATUSSUBSCRIBE_ADD"]={
	button1=ADD,
	button2=CANCEL,
	OnShow= function(self)
		self.button1:Disable()
	end,
	hasEditBox=true,
	EditBoxOnTextChanged = function(self)
		local text = self:GetText()
		StatusSubscribe_subscriptions = StatusSubscribe_subscriptions or {}
		for i,k in ipairs(StatusSubscribe_subscriptions) do
			if k == text then
				return
			end
		end
		if text ~= "" then
			self:GetParent().button1:Enable()
		end
	end,
	EditBoxOnEnterPressed = function(self)
		self:GetParent().button1:Click()
	end,
	OnAccept=function(self)
		local t = self.editBox:GetText()
		SendAddonMessage("StatusSubscribe","subrequest","WHISPER",t)
	end,
	text="Select a player to subscribe to:",
	timeout=0,
	whileDead=true,
	hideOnEscape=false,
}

StaticPopupDialogs["STATUSSUBSCRIBE_ADDED"]={
	button1=OKAY,
	text="|cffdf7800%s|r has confirmed your subscription.",
	showAlert=true,
	whileDead=true,
	hideOnEscape=true,
	timeout=0,
}

StaticPopupDialogs["STATUSSUBSCRIBE_DELETED"] = {
	button1=OKAY,
	text="|cffdf7800%s|r has deleted you from his subscriptions.",
	showAlert=true,
	whileDead=true,
	hideOnEscape=true,
	timeout=0,
}

StaticPopupDialogs["STATUSSUBSCRIBE_NOSUBSCRIBE"]={
	button1=OKAY,
	text="|cffdf7800%s|r has removed your subscription to him.",
	showAlert = true,
	whileDead=true,
	hideOnEscape = true,
	timeout = 0,
}

function StatusSubscribe:NotifySubscribers(msg)
	if not StatusSubscribe_subscribers then
		StatusSubscribe_subscribers = {}
	end
	for num, subscriber in ipairs(StatusSubscribe_subscribers) do
		if not InCombatLockdown() then
			dprint("Notifying "..subscriber.." of "..msg)
		end
		SendAddonMessage("StatusSubscribe", msg, "WHISPER", subscriber)
	end
end

function StatusSubscribe:IsSubscriber(cname)
	for num, name in ipairs(StatusSubscribe_subscribers) do
		if name == cname then
			return true
		end
	end
end

function StatusSubscribe:SendCompleteUpdate(rcvr)
	local function update(msg)
		SendAddonMessage("StatusSubscribe", msg, "WHISPER", rcvr)
	end
	if (not self.InEncounter) then return end
	update("engage|"..self.InEncounter.."|"..self.encounters[self.InEncounter].lname)
	update("phase|"..self.phase.."|"..self.encounters[self.InEncounter].phases[self.phase])
	for guid, data in pairs(self.healthcache) do
		update("health||"..guid.."|"..data)
	end
	for name, tdata in pairs(self.timercache) do
		if tdata.t:find("count") then
			update("timer|"..name.."|"..tdata.t.."|"..tdata.max)
			update("timerset|"..name.."|"..tdata.value)
		elseif tdata.t == "dcount" then
			update("timer|"..name.."|dcount|"..(tdata.duration-(GetTime()-tdata.start)))
		end
	end
end

function StatusSubscribe:CreateTimer(e_id, name, type, value)
	if (not self.InEncounter) or self.InEncounter ~= e_id then return end
	self.timercache = self.timercache or {}
	if type == "utime" then
		self.timercache[name] = {t="utime",start=GetTime(),duration=value}
	elseif type == "dtime" then
		self.timercache[name] = {t="dtime",start=GetTime(),duration=value}
	elseif type == "dcount" then
		self.timercache[name] = {t="dcount",value=value,max=value}
	elseif type == "ucount" then
		self.timercache[name] = {t="ucount",value=0,max=value}
	end
	self:NotifySubscribers("timer|"..name.."|"..type.."|"..value)
end

function StatusSubscribe:TimerSet(e_id, name, value)
	if (not self.InEncounter) or self.InEncounter ~= e_id then return end
	if (self.timercache) then
		if (self.timercache[name].value == value) then return end
		self.timercache[name].value = value
	end
	self:NotifySubscribers("timerset|"..name.."|"..value)
end

function StatusSubscribe:GetTimerValue(e_id, name)
	if (not self.InEncounter) or (self.InEncounter ~= e_id) then return end
	if (self.timercache) and (self.timercache[name]) then
		return self.timercache[name].value
	end
end

function StatusSubscribe:KillTimer(e_id, name)
	if (not self.InEncounter) or self.InEncounter ~= e_id then return end
	if (self.timercache) then
		self.timercache[name] = nil
	end
	self:NotifySubscribers("timerdel|"..name)
end

function StatusSubscribe:EngageEncounter(e_id)
	dprint("Engaging "..e_id)
	if not self.encounters[e_id] then return end
	self:UnregisterAllEvents()
	self.InEncounter = e_id
	self:NotifySubscribers("engage|"..e_id.."|"..self.encounters[e_id].lname)
	self.healthcache = {}
	self.timercache = {}
	self.manacache = {}
	self.phase = nil
	self.wipecheck:Show()
	for event,f in pairs(self.encounters[e_id].hooks) do
		if event == "ENCOUNTER_ENGAGE" then
			f()
		elseif event == "ALL_OUT_OF_COMBAT" then
			self.OnAllOutOfCombat = f
		elseif not ((event == "UNIT_HEALTH") or (event == "UNIT_MANA") or (event == "COMBAT_LOG_EVENT_UNFILTERED") or (event == "CHAT_MSG_ADDON")) then
			self:RegisterEvent(event)
		end
	end
	self:RegisterEvent("UNIT_HEALTH")
	self:RegisterEvent("UNIT_MANA")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("CHAT_MSG_ADDON")
end

function StatusSubscribe:EnterPhase(e_id, id)
	if (not self.InEncounter) or self.InEncounter ~= e_id then return end
	if (not self.encounters[e_id]) or (not self.encounters[e_id].phases) or (not self.encounters[e_id].phases[id]) then return end
	if (self.phase ~= id) then
		self.phase = id
		--always send localized name for now. will check if there is a better solution later.
		self:NotifySubscribers("phase|"..id.."|"..self.encounters[e_id].phases[id])
	end
end

function StatusSubscribe:IsInPhase(e_id, id)
	if (not self.InEncounter) or (self.InEncounter ~= e_id) then return end
	if self.phase == id then return true
	end
end

function StatusSubscribe:DisengageEncounter(e_id)
	dprint("Disengaging "..e_id)
	if (not self.InEncounter) or self.InEncounter ~= e_id then return end
	self.InEncounter = nil
	self:UnregisterAllEvents()
	self.wipecheck:Hide()
	self.OnAllOutOfCombat = nil
	self:NotifySubscribers("disengage|"..e_id)
	for event in pairs(self.triggers) do
		self:RegisterEvent(event)
	end
	self:RegisterEvent("CHAT_MSG_ADDON")
end

function StatusSubscribe:GetNID(guid)
	return tonumber("0x"..strsub(guid,9,12))
end

function StatusSubscribe:OpenView()
	if not self.gui then
		self.gui = CreateFrame("Frame", nil, UIParent)
		local gui = self.gui
		gui:SetFrameStrata("DIALOG")
		gui:SetMovable(true)
		gui:SetPoint("CENTER")
		gui:SetHeight(250)
		gui:SetWidth(250)
		gui.tx = gui:CreateTexture()
		gui.tx:SetAllPoints()
		gui.tx:SetTexture(0.3,0.3,0.3,0.5)
		gui.titlebar = CreateFrame("Frame", nil, gui)
		gui.titlebar:SetPoint("TOPLEFT")
		gui.titlebar:SetPoint("TOPRIGHT")
		gui.titlebar:SetHeight(10)
		gui.titlebar:EnableMouse(true)
		gui.titlebar:SetScript("OnMouseDown", function() gui:StartMoving() end)
		gui.titlebar:SetScript("OnMouseUp", function() gui:StopMovingOrSizing() end)
		gui.titlebar.tx = gui.titlebar:CreateTexture()
		gui.titlebar.tx:SetAllPoints()
		gui.titlebar.tx:SetTexture(0,0,0.8,1)
		gui.titlebar.close = CreateFrame("Button", nil, gui.titlebar)
		gui.titlebar.close:RegisterForClicks("AnyUp")
		gui.titlebar.close:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
		gui.titlebar.close:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
		gui.titlebar.close:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
		gui.titlebar.close:SetScript("OnClick", function() gui:Hide() end)
		gui.titlebar.close:SetPoint("TOPRIGHT", 1.5, 1.5)
		gui.titlebar.close:SetHeight(13)
		gui.titlebar.close:SetWidth(13)
		gui.titlebar.close:SetHitRectInsets(3,3,3,3)
		--[[gui.menu = CreateFrame("Frame", nil, gui)
		gui.menu:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background",edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", edgeSize=16, insets={bottom=4,left=5,right=5,top=5},tile=1,tileSize=16})
		gui.menu.frame1 = CreateFrame("Frame", nil,--]]
		gui.titlebar.delete = CreateFrame("Button", nil, gui.titlebar)
		gui.titlebar.delete:RegisterForClicks("AnyUp")
		gui.titlebar.delete:SetNormalTexture("Interface\\Minimap\\UI-Minimap-ZoomOutButton-Up")
		gui.titlebar.delete:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomOutButton-Highlight")
		gui.titlebar.delete:SetPushedTexture("Interface\\Minimap\\UI-Minimap-ZoomOutButton-Down")
		gui.titlebar.delete:SetDisabledTexture("Interface\\Minimap\\UI-Minimap-ZoomOutButton-Disabled")
		gui.titlebar.delete:SetScript("OnClick", function() local dia = StaticPopup_Show("STATUSSUBSCRIBE_DELETE", StatusSubscribe:GetCurrentViewSubscription()) dia.data = gui.curview end)
		gui.titlebar.delete:SetPoint("RIGHT", gui.titlebar.close, "LEFT")
		gui.titlebar.delete:SetHeight(7)
		gui.titlebar.delete:SetWidth(7)
		gui.titlebar.add = CreateFrame("Button", nil, gui.titlebar)
		gui.titlebar.add:RegisterForClicks("AnyUp")
		gui.titlebar.add:SetNormalTexture("Interface\\Minimap\\UI-Minimap-ZoomInButton-Up")
		gui.titlebar.add:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomInButton-Highlight")
		gui.titlebar.add:SetPushedTexture("Interface\\Minimap\\UI-Minimap-ZoomInButton-Down")
		gui.titlebar.add:SetDisabledTexture("Interface\\Minimap\\UI-Minimap-ZoomInButton-Disabled")
		gui.titlebar.add:SetScript("OnClick", function() StaticPopup_Show("STATUSSUBSCRIBE_ADD") end)
		gui.titlebar.add:SetPoint("RIGHT", gui.titlebar.delete, "LEFT", -3, 0)
		gui.titlebar.add:SetHeight(7)
		gui.titlebar.add:SetWidth(7)
		gui.tabbar = CreateFrame("Frame", nil, gui)
		gui.tabbar:SetPoint("TOPLEFT", gui.titlebar, "BOTTOMLEFT")
		gui.tabbar:SetPoint("TOPRIGHT", gui.titlebar, "BOTTOMRIGHT")
		gui.tabbar:SetHeight(15)
		local enterf = function(self) self.tx:Show() end
		local leavef = function(self) self.tx:Hide() end
		gui.tabbar.tabs = setmetatable({}, {__index=function(t,v)
			local f = CreateFrame("Frame", nil, gui.tabbar)
			if v > 1 then
				f:SetPoint("TOPLEFT", t[v-1], "TOPRIGHT")
				f:SetPoint("BOTTOMLEFT", t[v-1], "BOTTOMRIGHT")
			else
				f:SetPoint("TOPLEFT", gui.tabbar, "TOPLEFT")
				f:SetPoint("BOTTOMLEFT", gui.tabbar, "BOTTOMLEFT")
			end
			f:EnableMouse(true)
			f:SetScript("OnMouseUp", function() StatusSubscribe:SetView(v) end)
			f.tx = f:CreateTexture()
			f.tx:SetAllPoints()
			f.tx:SetTexture(0.8,0.8,0.8,0.4)
			f.htx = f:CreateTexture()
			f.htx:SetAllPoints()
			f.htx:SetTexture(0.8,0.8,0.8,0.6)
			if not f:IsMouseOver() then
				f.tx:Hide()
			end
			f.htx:Hide()
			f:SetScript("OnEnter", enterf)
			f:SetScript("OnLeave", leavef)
			f.text = f:CreateFontString()
			f.text:SetFont("Fonts\\FRIZQT__.TTF", 28, "OUTLINE")
			f.text:SetTextColor(1,1,1)
			f.text:SetPoint("TOPLEFT", 1, -1)
			f.text:SetPoint("BOTTOMRIGHT", -1, 1)
			function f:UpdateRenderedTextSize()
				local sh, sw = self.text:GetStringHeight(), self.text:GetStringWidth()
				local towidth = f:GetWidth()
				if towidth > 0 then
					local scale = towidth/sw
					self.text:SetTextHeight(math.min(sh*scale,13))
				end
			end
			function f:SetText(...)
				self.text:SetText(...)
				self:UpdateRenderedTextSize()
			end
			local sw = f.SetWidth
			function f:SetWidth(...)
				sw(self,...)
				self:UpdateRenderedTextSize()
			end
			t[v] = f
			return f
		end})
		function self:UpdateSubscriptionTabs()
			if not StatusSubscribe_subscriptions then
				StatusSubscribe_subscriptions = {}
			end
			local t = StatusSubscribe_subscriptions
			for _,f in ipairs(gui.tabbar.tabs) do
				f:Hide()
			end
			for c=1, #t do
				gui.tabbar.tabs[c]:SetText(t[c])
				gui.tabbar.tabs[c]:SetWidth(250/#t)
				gui.tabbar.tabs[c]:Show()
				gui.tabbar.tabs[c].htx:Hide()
			end
			if gui.curview > 0 then
				gui.tabbar.tabs[gui.curview].htx:Show()
			end
		end
		gui.viewscreen = CreateFrame("Frame", nil, gui)
		gui.viewscreen:SetPoint("TOPLEFT", gui.tabbar, "BOTTOMLEFT")
		gui.viewscreen:SetPoint("BOTTOMRIGHT")
		gui.viewscreen.tx = gui.viewscreen:CreateTexture()
		gui.viewscreen.tx:SetAllPoints()
		gui.viewscreen.tx:SetTexture(0.8,0.8,0.8,0.6)
		gui.viewscreen.tx:Hide()
		gui.viewscreen.na = gui.viewscreen:CreateFontString()
		gui.viewscreen.na:SetFont("Fonts\\FRIZQT__.TTF", 14, "THICKOUTLINE")
		gui.viewscreen.na:SetTextColor(0.8,0,0)
		gui.viewscreen.na:SetText("No subscription selected")
		gui.viewscreen.na:SetPoint("CENTER")
		gui.viewscreen.ne = gui.viewscreen:CreateFontString()
		gui.viewscreen.ne:SetFont("Fonts\\FRIZQT__.TTF", 14, "THICKOUTLINE")
		gui.viewscreen.ne:SetTextColor(0.8,0,0)
		gui.viewscreen.ne:SetText("No encounter in progress")
		gui.viewscreen.ne:SetPoint("CENTER")
		gui.viewscreen.data = CreateFrame("Frame", nil, gui.viewscreen)
		gui.viewscreen.data:SetAllPoints()
		gui.viewscreen.data.encounter = CreateFrame("Frame", nil, gui.viewscreen.data)
		gui.viewscreen.data.encounter.title = gui.viewscreen.data.encounter:CreateFontString()
		gui.viewscreen.data.encounter.title:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
		gui.viewscreen.data.encounter.title:SetTextColor(1,1,1)
		gui.viewscreen.data.encounter.title:SetText("Encounter info")
		gui.viewscreen.data.encounter.title:SetJustifyH("LEFT")
		gui.viewscreen.data.encounter.title:SetPoint("TOPLEFT", gui.viewscreen.data, "TOPLEFT",1,-1)
		gui.viewscreen.data.encounter.title:SetPoint("TOPRIGHT", gui.viewscreen.data, "TOPRIGHT", -1, -1)
		gui.viewscreen.data.encounter:SetPoint("TOPLEFT", gui.viewscreen.data.encounter.title, "TOPLEFT", -1, 1)
		gui.viewscreen.data.encounter:SetPoint("TOPRIGHT", gui.viewscreen.data.encounter.title, "TOPRIGHT", 1, 1)
		gui.viewscreen.data.encounter.namel = gui.viewscreen.data.encounter:CreateFontString()
		gui.viewscreen.data.encounter.name = gui.viewscreen.data.encounter:CreateFontString()
		gui.viewscreen.data.encounter.namel:SetPoint("TOPLEFT", gui.viewscreen.data.encounter.title, "BOTTOMLEFT", 0, -2)
		gui.viewscreen.data.encounter.name:SetPoint("TOPRIGHT", gui.viewscreen.data.encounter.title, "BOTTOMRIGHT", 0, -2)
		gui.viewscreen.data.encounter.namel:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE")
		gui.viewscreen.data.encounter.namel:SetTextColor(1,1,1)
		gui.viewscreen.data.encounter.namel:SetText("Encounter name")
		gui.viewscreen.data.encounter.name:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE")
		gui.viewscreen.data.encounter.name:SetTextColor(0.8,0,0)
		gui.viewscreen.data.encounter.name:SetPoint("LEFT", gui.viewscreen.data.encounter.namel, "RIGHT")
		gui.viewscreen.data.encounter.namel:SetWidth(125)
		gui.viewscreen.data.encounter.namel:SetJustifyH("LEFT")
		gui.viewscreen.data.encounter.name:SetJustifyH("RIGHT")
		gui.viewscreen.data.encounter.phasel = gui.viewscreen.data.encounter:CreateFontString()
		gui.viewscreen.data.encounter.phase = gui.viewscreen.data.encounter:CreateFontString()
		gui.viewscreen.data.encounter.phasel:SetPoint("TOPLEFT", gui.viewscreen.data.encounter.namel, "BOTTOMLEFT", 0, -1)
		gui.viewscreen.data.encounter.phase:SetPoint("TOPRIGHT", gui.viewscreen.data.encounter.name, "BOTTOMRIGHT", 0, -1)
		gui.viewscreen.data.encounter.phasel:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE")
		gui.viewscreen.data.encounter.phasel:SetTextColor(1,1,1)
		gui.viewscreen.data.encounter.phasel:SetText("Phase")
		gui.viewscreen.data.encounter.phase:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE")
		gui.viewscreen.data.encounter.phase:SetTextColor(0.8,0,0)
		gui.viewscreen.data.encounter.phase:SetPoint("LEFT", gui.viewscreen.data.encounter.phasel, "RIGHT")
		gui.viewscreen.data.encounter.phasel:SetWidth(125)
		gui.viewscreen.data.encounter.phasel:SetJustifyH("LEFT")
		gui.viewscreen.data.encounter.phase:SetJustifyH("RIGHT")
		gui.viewscreen.data.encounter:SetPoint("BOTTOMRIGHT", gui.viewscreen.data.encounter.phase, "BOTTOMRIGHT", 1, -1)
		gui.viewscreen.data.health = CreateFrame("Frame", nil, gui.viewscreen.data)
		gui.viewscreen.data.health.title = gui.viewscreen.data.health:CreateFontString()
		gui.viewscreen.data.health.title:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
		gui.viewscreen.data.health.title:SetTextColor(1,1,1)
		gui.viewscreen.data.health.title:SetText("Health info")
		gui.viewscreen.data.health.title:SetJustifyH("LEFT")
		gui.viewscreen.data.health.title:SetPoint("TOPLEFT", gui.viewscreen.data.encounter,"BOTTOMLEFT",1,-3)
		gui.viewscreen.data.health.title:SetPoint("TOPRIGHT", gui.viewscreen.data.encounter, "BOTTOMRIGHT", -1, -3)
		gui.viewscreen.data.health:SetPoint("TOPLEFT",gui.viewscreen.data.health.title,"TOPLEFT",-1,1)
		gui.viewscreen.data.health:SetPoint("TOPRIGHT",gui.viewscreen.data.health.title,"TOPRIGHT",1,1)
		gui.viewscreen.data.health.dataz = setmetatable({}, {__index=function(t,v)
			local f = CreateFrame("Frame", nil, gui.viewscreen.data.health)
			f.name = f:CreateFontString()
			f.name:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE")
			f.name:SetTextColor(1,1,1)
			f.health = f:CreateFontString()
			f.health:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE")
			f.health:SetTextColor(0.8,0,0)
			if v > 1 then
				f.name:SetPoint("TOPLEFT", t[v-1], "BOTTOMLEFT", 1, -1)
				f.health:SetPoint("TOPRIGHT", t[v-1], "BOTTOMRIGHT", -1, -1)
			else
				f.name:SetPoint("TOPLEFT", gui.viewscreen.data.health.title, "BOTTOMLEFT", 1, -2)
				f.health:SetPoint("TOPRIGHT", gui.viewscreen.data.health.title, "BOTTOMRIGHT", -1, -2)
			end
			f.health:SetPoint("LEFT", f.name, "RIGHT")
			f.name:SetWidth(200)
			f.name:SetJustifyH("LEFT")
			f.health:SetJustifyH("RIGHT")
			f:SetPoint("TOPLEFT", f.name, "TOPLEFT", -1, 1)
			f:SetPoint("BOTTOMRIGHT", f.health, "BOTTOMRIGHT", 1, -1)
			t[v] = f
			return f
		end})
		gui.viewscreen.data.health:SetPoint("BOTTOM", gui.viewscreen.data.health.title, "BOTTOM")
		function gui.viewscreen.data.health:UpdateHeight()
			self:ClearAllPoints()
			self:SetPoint("TOPLEFT",self.title,"TOPLEFT",-1,1)
			self:SetPoint("TOPRIGHT",self.title,"TOPRIGHT",1,1)
			local cnt = 1
			while (rawget(self.dataz,cnt)) and (self.dataz[cnt]:IsShown()) do
				cnt = cnt+1
			end
			if cnt > 1 then
				self:SetPoint("BOTTOM", self.dataz[cnt-1], "BOTTOM")
			else
				self:SetPoint("BOTTOM", self.title, "BOTTOM", -1, -1)
			end
		end	
		gui.viewscreen.data.timers = CreateFrame("Frame", nil, gui.viewscreen.data)
		gui.viewscreen.data.timers.title = gui.viewscreen.data.timers:CreateFontString()
		gui.viewscreen.data.timers.title:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
		gui.viewscreen.data.timers.title:SetTextColor(1,1,1)
		gui.viewscreen.data.timers.title:SetText("Timers")
		gui.viewscreen.data.timers.title:SetJustifyH("LEFT")
		gui.viewscreen.data.timers.title:SetPoint("TOPLEFT", gui.viewscreen.data.health, "BOTTOMLEFT", 1, -3)
		gui.viewscreen.data.timers.title:SetPoint("TOPRIGHT", gui.viewscreen.data.health, "BOTTOMRIGHT", -1, -3)
		gui.viewscreen.data.timers:SetPoint("TOPLEFT", gui.viewscreen.data.timers.title, "TOPLEFT", -1, 1)
		gui.viewscreen.data.timers:SetPoint("TOPRIGHT", gui.viewscreen.data.timers.title, "TOPRIGHT", 1, 1)
		gui.viewscreen.data.timers.dataz = setmetatable({}, {__index=function(t,v)
			local f = CreateFrame("Frame", nil, gui.viewscreen.data.timers)
			f.name = f:CreateFontString()
			f.name:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE")
			f.timer = f:CreateFontString()
			f.timer:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE")
			f.timer:SetTextColor(0.8,0,0)
			if v > 1 then
				f.name:SetPoint("TOPLEFT", t[v-1], "BOTTOMLEFT", 1, -1)
				f.timer:SetPoint("TOPRIGHT", t[v-1], "BOTTOMRIGHT", 1, -1)
			else
				f.name:SetPoint("TOPLEFT", gui.viewscreen.data.timers.title, "BOTTOMLEFT", 1, -2)
				f.timer:SetPoint("TOPRIGHT", gui.viewscreen.data.timers.title, "BOTTOMRIGHT", -1, -2)
			end
			f.timer:SetPoint("LEFT", f.name, "RIGHT")
			f.name:SetWidth(200)
			f.name:SetJustifyH("LEFT")
			f.timer:SetJustifyH("RIGHT")
			f.timer.update = CreateFrame("Frame", nil, f)
			f.timer.update.p = f.timer
			f.timer.update:Hide()
			f.timer.update:SetScript("OnUpdate", function(self)
				if not self.data then return end
				if not (self.data.t:find("time")) then return end
				local elapsed = round(GetTime()-self.data.start,0)
				if elapsed ~= self.cache then
					self.cache = elapsed
					local showt
					if self.data.t == "utime" then
						showt = elapsed
					elseif self.data.t == "dtime" then
						showt = math.max(self.data.duration-elapsed,0)
					end
					local min, sec = (math.floor(showt/60)),(showt%60)
					if min < 10 then min = "0"..min end
					if sec < 10 then sec = "0"..sec end
					showt = min..":"..sec
					self.p:SetText(showt)
				end
			end)
			f:SetPoint("TOPLEFT", f.name, "TOPLEFT", -1, 1)
			f:SetPoint("BOTTOMRIGHT", f.timer, "BOTTOMRIGHT", 1, -1)
			t[v] = f
			return f
		end})
		gui.viewscreen.data.timers:SetPoint("BOTTOM", gui.viewscreen.data.timers.title, "BOTTOM",1,-1)
		function gui.viewscreen.data.timers:UpdateHeight()
			self:ClearAllPoints()
			self:SetPoint("TOPLEFT",self.title,"TOPLEFT",-1,1)
			self:SetPoint("TOPRIGHT",self.title,"TOPRIGHT",1,1)
			local cnt = 1
			while (rawget(self.dataz,cnt)) and (self.dataz[cnt]:IsShown()) do
				cnt = cnt+1
			end
			if cnt > 1 then
				self:SetPoint("BOTTOM", self.dataz[cnt-1], "BOTTOM")
			else
				self:SetPoint("BOTTOM", self.title, "BOTTOM", 1, -1)
			end
		end
		self:SetView(0)
	else
		self.gui:Show()
	end
	self:UpdateSubscriptionTabs()
end

function StatusSubscribe:SetView(viewid)
	viewid = viewid or 0
	local gui = self.gui
	gui.curview = viewid
	if viewid > 0 then
		gui.titlebar.delete:Enable()
	else
		gui.titlebar.delete:Disable()
	end
	self:UpdateSubscriptionTabs()
	self:UpdateView()
end

function StatusSubscribe:GetCurrentViewSubscription()
	if not self.gui then return end
	if not StatusSubscribe_subscriptions then StatusSubscribe_subscriptions = {} end
	return StatusSubscribe_subscriptions[self.gui.curview]
end

function StatusSubscribe:UpdateView()
	local viewscreen = self.gui.viewscreen
	local cur = self.gui.curview
	if cur == 0 then
		viewscreen.tx:Hide()
		viewscreen.data:Hide()
		viewscreen.na:Show()
		viewscreen.ne:Hide()
	else
		local edata = self.EncounterStatus[self:GetCurrentViewSubscription()]
		if (not edata) or (not edata.encounter) then
			viewscreen.data:Hide() viewscreen.tx:Show() viewscreen.na:Hide() viewscreen.ne:Show()
		else
			viewscreen.data:Show()
			viewscreen.tx:Show()
			viewscreen.na:Hide()
			viewscreen.ne:Hide()
			-- update encounter data
			viewscreen.data.encounter.name:SetText(self.encounters[edata.encounter] and self.encounters[edata.encounter].lname or self.uencounters[edata.encounter])
			if edata.phase == "" then
				viewscreen.data.encounter.phase:SetText(edata.lphase)
			else
				viewscreen.data.encounter.phase:SetText(edata.lphase.." ("..edata.phase..")")
			end
			-- update health
			for _, f in ipairs(viewscreen.data.health.dataz) do
				f:Hide()
			end
			local c = 1
			for guid, data in pairs(edata.health) do
				local t = viewscreen.data.health.dataz[c]
				t.health:SetText(data.h)
				t.name:SetText(data.n)
				t:Show()
				c=c+1
			end
			viewscreen.data.health:UpdateHeight()
			-- update timers
			for _, f in ipairs(viewscreen.data.timers.dataz) do
				f:Hide()
				f.timer.update:Hide()
			end
			local c = 1
			for name, data in pairs(edata.timers) do
				local t = viewscreen.data.timers.dataz[c]
				t.name:SetText(name)
				if (data.t == "ucount") or (data.t == "dcount") then
					t.timer:SetText(data.value.."/"..data.max)
				else
					t.timer.update.data = data
					t.timer.update:Show()
					--local min, sec = (round(data.duration/60)),(data.duration%60)
					--if min < 10 then min = "0"..min end
					--if sec < 10 then sec = "0"..sec end
					--t.timer.v = min..":"..sec
				end
				t:Show()
				c=c+1
			end
			viewscreen.data.timers:UpdateHeight()
		end
	end
end

StatusSubscribe:SetScript("OnEvent", function(self, event, ...)
	if event == "ADDON_LOADED" then
		if (StatusSubscribe_subscribers and StatusSubscribe_subscriptions) then
			for num, name in ipairs(StatusSubscribe_subscriptions) do
				SendAddonMessage("StatusSubscribe", "request_update", "WHISPER", name)
			end
		end
	end
	if event == "CHAT_MSG_ADDON" then
		local prefix, message, chan, sender = ...
		if prefix ~= "StatusSubscribe" then return end
		local args = {strsplit("|",message)}
		local action = args[1]
		tremove(args,1)
		if not self.EncounterStatus[sender] then
			self.EncounterStatus[sender] = {health={},mana={},phase=0,lphase=self.locale.Undefined,timers={}}
			if self:GetCurrentViewSubscription() == sender then
				self:UpdateView()
			end
		end
		local estatus = self.EncounterStatus[sender]
		if action == "engage" then
			local id, lname = args[1],args[2]
			if not self.encounters[id] then
				dprint("Getting unknown encounter "..lname..".")
				self.uencounters[id] = lname
			end
			estatus.encounter = id
			if self:GetCurrentViewSubscription() == sender then
				self:UpdateView()
			end
			print("|cffaaaaaa-StatusSubscribe-|r "..sender.." has engaged "..(self.encounters[id] and self.encounters[id].lname or self.uencounters[id]).."!")
		elseif action == "disengage" then
			local id = args[1]
			if not (self.encounters[id] or self.uencounters[id]) then return end
			if not estatus.encounter == id then return end
			--reset everything related to the encounter
			estatus.encounter = nil
			estatus.health = {}
			estatus.mana = {}
			estatus.phase = 0
			estatus.lphase = self.locale.Undefined
			estatus.timers = {}
			if self:GetCurrentViewSubscription() == sender then
				self:UpdateView()
			end
		elseif action == "phase" then
			local pid, plname = args[1],args[2]
			if (not estatus.encounter) or (estatus.phase == pid) then return end
			estatus.phase = pid
			estatus.lphase = plname
			if self:GetCurrentViewSubscription() == sender then
				self:UpdateView()
			end
		elseif action == "health" then
			local name, guid, hp = args[1],args[2],args[3]
			if (not estatus.encounter) then return end
			if hp == "remove" then
				estatus.health[guid] = nil
				estatus.mana[guid] = nil
			else
				estatus.health[guid] = {n=name,h=hp}
			end
			if self:GetCurrentViewSubscription() == sender then
				self:UpdateView()
			end
		elseif action == "mana" then
			local guid, mana = args[1],args[2]
			if (not estatus.encounter) then return end
			estatus.mana[guid] = mana
			if self:GetCurrentViewSubscription() == sender then
				self:UpdateView()
			end
		elseif action == "timer" then
			local name,type,value = args[1],args[2],args[3]
			if type == "utime" then
				estatus.timers[name] = {t="utime",start=GetTime(),duration=value}
			elseif type == "dtime" then
				estatus.timers[name] = {t="dtime",start=GetTime(),duration=value}
			elseif type == "dcount" then
				estatus.timers[name] = {t="dcount",value=value,max=value}
			elseif type == "ucount" then
				estatus.timers[name] = {t="ucount",value=0,max=value}
			end
			if self:GetCurrentViewSubscription() == sender then
				self:UpdateView()
			end
		elseif action == "timerset" then
			local name,value = args[1],args[2]
			if (not estatus.timers[name]) or (estatus.timers[name].t:find("time")) then return end
			estatus.timers[name].value = value
			if self:GetCurrentViewSubscription() == sender then
				self:UpdateView()
			end
		elseif action == "timerdel" then
			local name = args[1]
			if (not estatus.timers[name]) then return end
			estatus.timers[name] = nil
			if self:GetCurrentViewSubscription() == sender then
				self:UpdateView()
			end
		elseif action == "subrequest" then
			local dia = StaticPopup_Show("STATUSSUBSCRIBE_INC", sender)
			dia.data = sender
		elseif action == "subconfirm" then
			StatusSubscribe_subscriptions = StatusSubscribe_subscriptions or {}
			tinsert(StatusSubscribe_subscriptions, sender)
			StaticPopup_Show("STATUSSUBSCRIBE_ADDED", sender)
			StatusSubscribe:UpdateSubscriptionTabs()
		elseif action == "subdelete" then
			for num,name in ipairs(StatusSubscribe_subscribers) do
				if name == sender then
					tremove(StatusSubscribe_subscribers, num)
				end
				StaticPopup_Show("STATUSSUBSCRIBE_DELETED", sender)
			end
		elseif action == "request_update" then
			if self:IsSubscriber(sender) then
				self:SendCompleteUpdate(sender)
			else
				SendAddonMessage("StatusSubscribe","nosubscribe","WHISPER",sender)
			end
		elseif action == "nosubscribe" then
			for num, name in ipairs(StatusSubscribe_subscriptions) do
				if name == sender then
					tremove(StatusSubscribe_subscriptions, num)
				end
				if (StatusSubscribe.gui) and (StatusSubscribe.gui.curview == num) then
					StatusSubscribe:SetView(0)
				end
				StaticPopup_Show("STATUSSUBSCRIBE_NOSUBSCRIBE", sender)
				StatusSubscribe:UpdateSubscriptionTabs()
			end
		end --add any other msgs as elseif above this
		return
	end
	if not self.InEncounter then
		if self.triggers[event] then
			for num, id in ipairs(self.triggers[event]) do
				local shouldEngage = self.encounters[id].engage(event,...)
				if shouldEngage then
					self:EngageEncounter(id)
					return
				end
			end
		end
	else
		local id = self.InEncounter
		if event == "UNIT_HEALTH" then
			local name, guid = (UnitName(...)),UnitGUID(...)
			local nid = self:GetNID(guid)
			if (self.encounters[id].health[nid] ~= nil) then
				local hp = round((UnitHealth(...)/UnitHealthMax(...))*100,1)
				if hp == 0 then
					if self.encounters[self.InEncounter].health[nid] then
						hp = "DEAD"
					else
						hp = "remove"
					end
				else
					hp = hp.."%"
				end
				if (not self.healthcache[guid]) or (self.healthcache[guid] ~= hp) then
					self.healthcache[guid] = hp
					self:NotifySubscribers("health|"..name.."|"..guid.."|"..hp)
				end
			end
		elseif event == "UNIT_MANA" then
			if not (self.encounters[id].mana) then return end
			local guid = UnitGUID(...)
			if (self.encounters[id].mana[self:GetNID(guid)]) then
				local mana = round((UnitPower(...)/UnitPowerMax(...))*100,1).."%"
				if (not self.manacache[guid]) or (self.manacache[guid] ~= mana) then
					self.manacache[guid] = mana
					self:NotifySubscribers("mana|"..guid.."|"..mana)
				end
			end
		elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
			local event, name, guid = (select(2, ...)), (select(7, ...)), (select(6,...))
			if event == "UNIT_DIED" then
				local nid = self:GetNID(guid)
				if self.encounters[id].health[nid] == true then
					self.healthcache[guid] = "DEAD"
					self:NotifySubscribers("health|"..name.."|"..guid.."|DEAD")
				elseif self.encounters[id].health[nid] == false then
					self.healthcache[guid] = nil
					self:NotifySubscribers("health|"..name.."|"..guid.."|remove")
				end
			end
		end
		if self.encounters[id].hooks[event] then
			self.encounters[id].hooks[event](...)
		end
	end
end)
StatusSubscribe:RegisterEvent("ADDON_LOADED")
StatusSubscribe:RegisterEvent("CHAT_MSG_ADDON")

SlashCmdList["STATUSSUBSCRIBE"] = function() StatusSubscribe:OpenView() end
SLASH_STATUSSUBSCRIBE1 = "/statussubscribe"
SLASH_STATUSSUBSCRIBE2 = "/status"